# pylint:disable=line-too-long
"""
The tool to check the availability or syntax of domains, IPv4, IPv6 or URL.

::


    ██████╗ ██╗   ██╗███████╗██╗   ██╗███╗   ██╗ ██████╗███████╗██████╗ ██╗     ███████╗
    ██╔══██╗╚██╗ ██╔╝██╔════╝██║   ██║████╗  ██║██╔════╝██╔════╝██╔══██╗██║     ██╔════╝
    ██████╔╝ ╚████╔╝ █████╗  ██║   ██║██╔██╗ ██║██║     █████╗  ██████╔╝██║     █████╗
    ██╔═══╝   ╚██╔╝  ██╔══╝  ██║   ██║██║╚██╗██║██║     ██╔══╝  ██╔══██╗██║     ██╔══╝
    ██║        ██║   ██║     ╚██████╔╝██║ ╚████║╚██████╗███████╗██████╔╝███████╗███████╗
    ╚═╝        ╚═╝   ╚═╝      ╚═════╝ ╚═╝  ╚═══╝ ╚═════╝╚══════╝╚═════╝ ╚══════╝╚══════╝

Provides our exceptions.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Special thanks:
    https://pyfunceble.github.io/special-thanks.html

Contributors:
    https://pyfunceble.github.io/contributors.html

Project link:
    https://github.com/funilrys/PyFunceble

Project documentation:
    https://pyfunceble.readthedocs.io/en/dev/

Project homepage:
    https://pyfunceble.github.io/

License:
::


    MIT License

    Copyright (c) 2017, 2018, 2019, 2020 Nissar Chababy

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""


class PyFuncebleException(Exception):
    """
    Describes our own exceptions.
    """


class PyFuncebleExternalException(PyFuncebleException):
    """
    Describes an exception which is caused by an external input.
    """


class PyFuncebleInternalException(PyFuncebleException):
    """
    Describes an exception which is caused by our own logic.
    """


class WrongParameterType(PyFuncebleInternalException):
    """
    Describes a wrong parameter type.
    """


class WrongParameterValue(PyFuncebleInternalException):
    """
    Describes a wrong parameter value.
    """


class NoInternetConnection(PyFuncebleExternalException):
    """
    Describes a missing connection.
    """


class ConfigurationFileNotFound(PyFuncebleExternalException):
    """
    Describes a missing configuration file.
    """


class GitHubTokenNotFound(PyFuncebleExternalException):
    """
    Describes a missing GitHub token.
    """


class GitLabTokenNotFound(PyFuncebleExternalException):
    """
    Describes a missing GitLab token.
    """


class GitEmailNotFound(PyFuncebleExternalException):
    """
    Describes a missing Git Email.
    """


class GitNameNotFound(PyFuncebleExternalException):
    """
    Describes a missing Git Name.
    """


class PleaseUpdatePyFunceble(PyFuncebleInternalException):
    """
    Describes the impossiblity to continue with an older version.
    """


class NoConversionMade(PyFuncebleInternalException):
    """
    Describes the fact that a conversion was expected but none
    was made.
    """


class NoExtractionMade(PyFuncebleInternalException):
    """
    Describes the fact that an extraction was expected but none
    was made.
    """


class UnknownSubject(PyFuncebleInternalException):
    """
    Describes the fact that an unknown subject is inputed.
    """


class NoDownloadDestinationGiven(PyFuncebleInternalException):
    """
    Describes the fact that the download destination was not declared.
    """


class NoDownloadLinkGiven(PyFuncebleInternalException):
    """
    Describes the fact that no download link was declared.
    """


class UserAgentNotFound(PyFuncebleInternalException):
    """
    Describes the fact that we could not find a user
    agent to work with.
    """


class UserAgentBrowserNotFound(PyFuncebleInternalException):
    """
    Describes the fact that we could not find a valid
    browser to work with.
    """


class UserAgentPlatformNotFound(PyFuncebleInternalException):
    """
    Describes the fact that we could not find a valid
    browser to work with.
    """
